@@ -7,7 +7,9 @@ var searchData=
   ['libinput_5fevent_5fdevice_5fnotify',['libinput_event_device_notify',['../structlibinput__event__device__notify.html',1,'']]],
   ['libinput_5fevent_5fkeyboard',['libinput_event_keyboard',['../structlibinput__event__keyboard.html',1,'']]],
   ['libinput_5fevent_5fpointer',['libinput_event_pointer',['../structlibinput__event__pointer.html',1,'']]],
+  ['libinput_5fevent_5ftablet_5ftool',['libinput_event_tablet_tool',['../structlibinput__event__tablet__tool.html',1,'']]],
   ['libinput_5fevent_5ftouch',['libinput_event_touch',['../structlibinput__event__touch.html',1,'']]],
   ['libinput_5finterface',['libinput_interface',['../structlibinput__interface.html',1,'']]],
-  ['libinput_5fseat',['libinput_seat',['../structlibinput__seat.html',1,'']]]
+  ['libinput_5fseat',['libinput_seat',['../structlibinput__seat.html',1,'']]],
+  ['libinput_5ftablet_5ftool',['libinput_tablet_tool',['../structlibinput__tablet__tool.html',1,'']]]
 ];
