@@ -94,7 +94,7 @@ var searchData=
   ['libinput_5fevent_5fget_5fcontext',['libinput_event_get_context',['../group__event.html#ga28557ae0295f61c45f9b6e4ff7f86632',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5fdevice',['libinput_event_get_device',['../group__event.html#ga8a49a089181960f7536344f7637ea777',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5fdevice_5fnotify_5fevent',['libinput_event_get_device_notify_event',['../group__event.html#ga2bbc1391a3397d57d37ae3e2f8175abc',1,'libinput.h']]],
-  ['libinput_5fevent_5fget_5fgesture_5fevent',['libinput_event_get_gesture_event',['../libinput_8h.html#ae173f30b70c265036bbdc9557f3115c4',1,'libinput.h']]],
+  ['libinput_5fevent_5fget_5fgesture_5fevent',['libinput_event_get_gesture_event',['../group__event.html#gae173f30b70c265036bbdc9557f3115c4',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5fkeyboard_5fevent',['libinput_event_get_keyboard_event',['../group__event.html#ga8378f8fdd01e8afbcea8ddbb5411570d',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5fpointer_5fevent',['libinput_event_get_pointer_event',['../group__event.html#ga52acdc3afcf75efeb8153f50eef20615',1,'libinput.h']]],
   ['libinput_5fevent_5fget_5ftablet_5ftool_5fevent',['libinput_event_get_tablet_tool_event',['../group__event.html#gadfd7d9f1c3210f394faa60928f3c187f',1,'libinput.h']]],
@@ -112,7 +112,7 @@ var searchData=
   ['libinput_5fevent_5fpointer_5fget_5fabsolute_5fy_5ftransformed',['libinput_event_pointer_get_absolute_y_transformed',['../group__event__pointer.html#gaa48effb21f132f05f280859f3d2a9f23',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5faxis_5fsource',['libinput_event_pointer_get_axis_source',['../group__event__pointer.html#ga2116f4bbedb61532e71d16c4f87bd4ca',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5faxis_5fvalue',['libinput_event_pointer_get_axis_value',['../group__event__pointer.html#ga81ad7d8a95c456731a874e584c4c8dda',1,'libinput.h']]],
-  ['libinput_5fevent_5fpointer_5fget_5faxis_5fvalue_5fdiscrete',['libinput_event_pointer_get_axis_value_discrete',['../libinput_8h.html#a38d12fc6884c9943e261febdb2384b98',1,'libinput.h']]],
+  ['libinput_5fevent_5fpointer_5fget_5faxis_5fvalue_5fdiscrete',['libinput_event_pointer_get_axis_value_discrete',['../group__event__pointer.html#ga38d12fc6884c9943e261febdb2384b98',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5fbase_5fevent',['libinput_event_pointer_get_base_event',['../group__event__pointer.html#ga4d4f75669001c323dcdc3cc9aec1574a',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5fbutton',['libinput_event_pointer_get_button',['../group__event__pointer.html#ga6dc942e706a29d342a2021afaa49548b',1,'libinput.h']]],
   ['libinput_5fevent_5fpointer_5fget_5fbutton_5fstate',['libinput_event_pointer_get_button_state',['../group__event__pointer.html#ga41e87521ab2a8b7783cb47a63d71da18',1,'libinput.h']]],
