@@ -27,7 +27,7 @@
 
 #include "evdev.h"
 
-#define MIDDLEBUTTON_TIMEOUT 50
+#define MIDDLEBUTTON_TIMEOUT ms2us(50)
 
 /*****************************************
  * BEFORE YOU EDIT THIS FILE, look at the state diagram in
@@ -43,8 +43,6 @@
  * as-is.
  */
 
-#define CASE_RETURN_STRING(a) case a: return #a;
-
 static inline const char*
 middlebutton_state_to_str(enum evdev_middlebutton_state state)
 {
@@ -612,7 +610,7 @@ evdev_middlebutton_filter_button(struct evdev_device *device,
 	if (button < BTN_LEFT ||
 	    bit >= sizeof(device->middlebutton.button_mask) * 8) {
 		log_bug_libinput(device->base.seat->libinput,
-				 "Button mask too small for %d\n",
+				 "Button mask too small for %s\n",
 				 libevdev_event_code_get_name(EV_KEY,
 							      button));
 		return true;
@@ -642,9 +640,7 @@ evdev_middlebutton_handle_timeout(uint64_t now, void *data)
 {
 	struct evdev_device *device = (struct evdev_device*)data;
 
-	evdev_middlebutton_handle_event(device,
-					libinput_now(device->base.seat->libinput),
-					MIDDLEBUTTON_EVENT_TIMEOUT);
+	evdev_middlebutton_handle_event(device, now, MIDDLEBUTTON_EVENT_TIMEOUT);
 }
 
 static int
