@@ -72,9 +72,9 @@ list_empty(const struct list *list)
 }
 
 void
-ratelimit_init(struct ratelimit *r, uint64_t ival_ms, unsigned int burst)
+ratelimit_init(struct ratelimit *r, uint64_t ival_us, unsigned int burst)
 {
-	r->interval = ival_ms;
+	r->interval = ival_us;
 	r->begin = 0;
 	r->burst = burst;
 	r->num = 0;
@@ -97,17 +97,17 @@ enum ratelimit_state
 ratelimit_test(struct ratelimit *r)
 {
 	struct timespec ts;
-	uint64_t mtime;
+	uint64_t utime;
 
 	if (r->interval <= 0 || r->burst <= 0)
 		return RATELIMIT_PASS;
 
 	clock_gettime(CLOCK_MONOTONIC, &ts);
-	mtime = ts.tv_sec * 1000 + ts.tv_nsec / 1000 / 1000;
+	utime = s2us(ts.tv_sec) + ns2us(ts.tv_nsec);
 
-	if (r->begin <= 0 || r->begin + r->interval < mtime) {
+	if (r->begin <= 0 || r->begin + r->interval < utime) {
 		/* reset counter */
-		r->begin = mtime;
+		r->begin = utime;
 		r->num = 1;
 		return RATELIMIT_PASS;
 	} else if (r->num < r->burst) {
@@ -234,3 +234,33 @@ parse_trackpoint_accel_property(const char *prop)
 
 	return accel;
 }
+
+/**
+ * Parses a simple dimension string in the form of "10x40". The two
+ * numbers must be positive integers in decimal notation.
+ * On success, the two numbers are stored in w and h. On failure, w and h
+ * are unmodified.
+ *
+ * @param prop The value of the property
+ * @param w Returns the first component of the dimension
+ * @param h Returns the second component of the dimension
+ * @return true on success, false otherwise
+ */
+bool
+parse_dimension_property(const char *prop, size_t *w, size_t *h)
+{
+	int x, y;
+
+	if (!prop)
+		return false;
+
+	if (sscanf(prop, "%dx%d", &x, &y) != 2)
+		return false;
+
+	if (x < 0 || y < 0)
+		return false;
+
+	*w = (size_t)x;
+	*h = (size_t)y;
+	return true;
+}
