@@ -97,6 +97,8 @@ int list_empty(const struct list *list);
 #define streq(s1, s2) (strcmp((s1), (s2)) == 0)
 #define strneq(s1, s2, n) (strncmp((s1), (s2), (n)) == 0)
 
+#define NCHARS(x) ((size_t)(((x) + 7) / 8))
+
 #ifdef DEBUG_TRACE
 #define debug_trace(...) \
 	do { \
@@ -106,6 +108,7 @@ int list_empty(const struct list *list);
 #else
 #define debug_trace(...) { }
 #endif
+
 #define LIBINPUT_EXPORT __attribute__ ((visibility("default")))
 
 static inline void *
@@ -114,6 +117,28 @@ zalloc(size_t size)
 	return calloc(1, size);
 }
 
+/* This bitfield helper implementation is taken from from libevdev-util.h,
+ * except that it has been modified to work with arrays of unsigned chars
+ */
+
+static inline int
+bit_is_set(const unsigned char *array, int bit)
+{
+    return !!(array[bit / 8] & (1 << (bit % 8)));
+}
+
+static inline void
+set_bit(unsigned char *array, int bit)
+{
+    array[bit / 8] |= (1 << (bit % 8));
+}
+
+static inline void
+clear_bit(unsigned char *array, int bit)
+{
+    array[bit / 8] &= ~(1 << (bit % 8));
+}
+
 static inline void
 msleep(unsigned int ms)
 {
@@ -123,19 +148,19 @@ msleep(unsigned int ms)
 static inline int
 long_bit_is_set(const unsigned long *array, int bit)
 {
-    return !!(array[bit / LONG_BITS] & (1LL << (bit % LONG_BITS)));
+	return !!(array[bit / LONG_BITS] & (1LL << (bit % LONG_BITS)));
 }
 
 static inline void
 long_set_bit(unsigned long *array, int bit)
 {
-    array[bit / LONG_BITS] |= (1LL << (bit % LONG_BITS));
+	array[bit / LONG_BITS] |= (1LL << (bit % LONG_BITS));
 }
 
 static inline void
 long_clear_bit(unsigned long *array, int bit)
 {
-    array[bit / LONG_BITS] &= ~(1LL << (bit % LONG_BITS));
+	array[bit / LONG_BITS] &= ~(1LL << (bit % LONG_BITS));
 }
 
 static inline void
@@ -259,6 +284,16 @@ matrix_to_farray6(const struct matrix *m, float out[6])
 	out[5] = m->val[1][2];
 }
 
+static inline void
+matrix_to_relative(struct matrix *dest, const struct matrix *src)
+{
+	matrix_init_identity(dest);
+	dest->val[0][0] = src->val[0][0];
+	dest->val[0][1] = src->val[0][1];
+	dest->val[1][0] = src->val[1][0];
+	dest->val[1][1] = src->val[1][1];
+}
+
 /**
  * Simple wrapper for asprintf that ensures the passed in-pointer is set
  * to NULL upon error.
