@@ -35,6 +35,28 @@
 #include <libinput.h>
 #include <math.h>
 
+void
+litest_fail_condition(const char *file,
+		      int line,
+		      const char *func,
+		      const char *condition,
+		      const char *message,
+		      ...);
+void
+litest_fail_comparison_int(const char *file,
+			   int line,
+			   const char *func,
+			   const char *operator,
+			   int a,
+			   int b,
+			   const char *astr,
+			   const char *bstr);
+void
+litest_fail_comparison_ptr(const char *file,
+			   int line,
+			   const char *func,
+			   const char *comparison);
+
 #define litest_assert(cond) \
 	do { \
 		if (!(cond)) \
@@ -111,6 +133,24 @@
 #define litest_assert_ptr_notnull(a_) \
 	litest_assert_comparison_ptr_(a_, !=, NULL)
 
+#define litest_assert_double_eq(a_, b_)\
+	ck_assert_int_eq((int)((a_) * 256), (int)((b_) * 256))
+
+#define litest_assert_double_ne(a_, b_)\
+	ck_assert_int_ne((int)((a_) * 256), (int)((b_) * 256))
+
+#define litest_assert_double_lt(a_, b_)\
+	ck_assert_int_lt((int)((a_) * 256), (int)((b_) * 256))
+
+#define litest_assert_double_le(a_, b_)\
+	ck_assert_int_le((int)((a_) * 256), (int)((b_) * 256))
+
+#define litest_assert_double_gt(a_, b_)\
+	ck_assert_int_gt((int)((a_) * 256), (int)((b_) * 256))
+
+#define litest_assert_double_ge(a_, b_)\
+	ck_assert_int_ge((int)((a_) * 256), (int)((b_) * 256))
+
 enum litest_device_type {
 	LITEST_NO_DEVICE = -1,
 	LITEST_SYNAPTICS_CLICKPAD = -2,
@@ -146,9 +186,16 @@ enum litest_device_type {
 	LITEST_MOUSE_WHEEL_CLICK_ANGLE = -32,
 	LITEST_APPLE_KEYBOARD = -33,
 	LITEST_ANKER_MOUSE_KBD = -34,
+	LITEST_WACOM_BAMBOO = -35,
+	LITEST_WACOM_CINTIQ = -36,
+	LITEST_WACOM_INTUOS = -37,
+	LITEST_WACOM_ISDV4 = -38,
+	LITEST_WALTOP = -39,
+	LITEST_HUION_TABLET = -40,
 	LITEST_CYBORG_RAT = -41,
 	LITEST_YUBIKEY = -42,
 	LITEST_SYNAPTICS_I2C = -43,
+	LITEST_WACOM_CINTIQ_24HD = -44,
 };
 
 enum litest_device_feature {
@@ -171,6 +218,10 @@ enum litest_device_feature {
 	LITEST_PROTOCOL_A = 1 << 14,
 	LITEST_HOVER = 1 << 15,
 	LITEST_ELLIPSE = 1 << 16,
+	LITEST_TABLET = 1 << 17,
+	LITEST_DISTANCE = 1 << 18,
+	LITEST_TOOL_SERIAL = 1 << 19,
+	LITEST_TILT = 1 << 20,
 };
 
 struct litest_device {
@@ -191,9 +242,26 @@ struct litest_device {
 
 struct axis_replacement {
 	int32_t evcode;
-	int32_t value;
+	double value;
 };
 
+static inline void
+litest_axis_set_value(struct axis_replacement *axes, int code, double value)
+{
+	litest_assert_double_ge(value, 0.0);
+	litest_assert_double_le(value, 100.0);
+
+	while (axes->evcode != -1) {
+		if (axes->evcode == code) {
+			axes->value = value;
+			return;
+		}
+		axes++;
+	}
+
+	litest_abort_msg("Missing axis code %d\n", code);
+}
+
 /* A loop range, resolves to:
    for (i = lower; i < upper; i++)
  */
@@ -206,28 +274,6 @@ struct libinput *litest_create_context(void);
 void litest_disable_log_handler(struct libinput *libinput);
 void litest_restore_log_handler(struct libinput *libinput);
 
-void
-litest_fail_condition(const char *file,
-		      int line,
-		      const char *func,
-		      const char *condition,
-		      const char *message,
-		      ...);
-void
-litest_fail_comparison_int(const char *file,
-			   int line,
-			   const char *func,
-			   const char *operator,
-			   int a,
-			   int b,
-			   const char *astr,
-			   const char *bstr);
-void
-litest_fail_comparison_ptr(const char *file,
-			   int line,
-			   const char *func,
-			   const char *comparison);
-
 #define litest_add(name_, func_, ...) \
 	_litest_add(name_, #func_, func_, __VA_ARGS__)
 #define litest_add_ranged(name_, func_, ...) \
@@ -240,38 +286,49 @@ litest_fail_comparison_ptr(const char *file,
 	_litest_add_no_device(name_, #func_, func_)
 #define litest_add_ranged_no_device(name_, func_, ...) \
 	_litest_add_ranged_no_device(name_, #func_, func_, __VA_ARGS__)
-void _litest_add(const char *name,
-		 const char *funcname,
-		 void *func,
-		 enum litest_device_feature required_feature,
-		 enum litest_device_feature excluded_feature);
-void _litest_add_ranged(const char *name,
-			const char *funcname,
-			void *func,
-			enum litest_device_feature required,
-			enum litest_device_feature excluded,
-			const struct range *range);
-void _litest_add_for_device(const char *name,
-			    const char *funcname,
-			    void *func,
-			    enum litest_device_type type);
-void _litest_add_ranged_for_device(const char *name,
-				   const char *funcname,
-				   void *func,
-				   enum litest_device_type type,
-				   const struct range *range);
-void _litest_add_no_device(const char *name,
-			   const char *funcname,
-			   void *func);
-void _litest_add_ranged_no_device(const char *name,
-				  const char *funcname,
-				  void *func,
-				  const struct range *range);
+void
+_litest_add(const char *name,
+	    const char *funcname,
+	    void *func,
+	    enum litest_device_feature required_feature,
+	    enum litest_device_feature excluded_feature);
+void
+_litest_add_ranged(const char *name,
+		   const char *funcname,
+		   void *func,
+		   enum litest_device_feature required,
+		   enum litest_device_feature excluded,
+		   const struct range *range);
+void
+_litest_add_for_device(const char *name,
+		       const char *funcname,
+		       void *func,
+		       enum litest_device_type type);
+void
+_litest_add_ranged_for_device(const char *name,
+			      const char *funcname,
+			      void *func,
+			      enum litest_device_type type,
+			      const struct range *range);
+void
+_litest_add_no_device(const char *name,
+		      const char *funcname,
+		      void *func);
+void
+_litest_add_ranged_no_device(const char *name,
+			     const char *funcname,
+			     void *func,
+			     const struct range *range);
 
-extern void litest_setup_tests(void);
-struct litest_device * litest_create_device(enum litest_device_type which);
-struct litest_device * litest_add_device(struct libinput *libinput,
-					 enum litest_device_type which);
+extern void
+litest_setup_tests(void);
+
+struct litest_device *
+litest_create_device(enum litest_device_type which);
+
+struct litest_device *
+litest_add_device(struct libinput *libinput,
+		  enum litest_device_type which);
 struct libevdev_uinput *
 litest_create_uinput_device_from_description(const char *name,
 					     const struct input_id *id,
@@ -291,155 +348,245 @@ litest_add_device_with_overrides(struct libinput *libinput,
 				 const struct input_absinfo *abs_override,
 				 const int *events_override);
 
-struct litest_device *litest_current_device(void);
-void litest_delete_device(struct litest_device *d);
-int litest_handle_events(struct litest_device *d);
+struct litest_device *
+litest_current_device(void);
+
+void
+litest_delete_device(struct litest_device *d);
+
+int
+litest_handle_events(struct litest_device *d);
+
+void
+litest_event(struct litest_device *t,
+	     unsigned int type,
+	     unsigned int code,
+	     int value);
+int
+litest_auto_assign_value(struct litest_device *d,
+			 const struct input_event *ev,
+			 int slot, double x, double y,
+			 struct axis_replacement *axes,
+			 bool touching);
+void
+litest_touch_up(struct litest_device *d, unsigned int slot);
+
+void
+litest_touch_move(struct litest_device *d,
+		  unsigned int slot,
+		  double x,
+		  double y);
+
+void
+litest_touch_move_extended(struct litest_device *d,
+			   unsigned int slot,
+			   double x,
+			   double y,
+			   struct axis_replacement *axes);
+
+void
+litest_touch_down(struct litest_device *d,
+		  unsigned int slot,
+		  double x,
+		  double y);
+
+void
+litest_touch_down_extended(struct litest_device *d,
+			   unsigned int slot,
+			   double x,
+			   double y,
+			   struct axis_replacement *axes);
+
+void
+litest_touch_move_to(struct litest_device *d,
+		     unsigned int slot,
+		     double x_from, double y_from,
+		     double x_to, double y_to,
+		     int steps, int sleep_ms);
+
+void
+litest_touch_move_two_touches(struct litest_device *d,
+			      double x0, double y0,
+			      double x1, double y1,
+			      double dx, double dy,
+			      int steps, int sleep_ms);
+
+void
+litest_touch_move_three_touches(struct litest_device *d,
+				double x0, double y0,
+				double x1, double y1,
+				double x2, double y2,
+				double dx, double dy,
+				int steps, int sleep_ms);
+
+void
+litest_tablet_proximity_in(struct litest_device *d,
+			   int x, int y,
+			   struct axis_replacement *axes);
+
+void
+litest_tablet_proximity_out(struct litest_device *d);
+
+void
+litest_tablet_motion(struct litest_device *d,
+		     int x, int y,
+		     struct axis_replacement *axes);
+
+void
+litest_hover_start(struct litest_device *d,
+		   unsigned int slot,
+		   double x,
+		   double y);
+
+void
+litest_hover_end(struct litest_device *d, unsigned int slot);
 
-void litest_event(struct litest_device *t,
-		  unsigned int type,
-		  unsigned int code,
-		  int value);
-int litest_auto_assign_value(struct litest_device *d,
-			     const struct input_event *ev,
-			     int slot, double x, double y,
-			     struct axis_replacement *axes,
-			     bool touching);
-void litest_touch_up(struct litest_device *d, unsigned int slot);
-void litest_touch_move(struct litest_device *d,
-		       unsigned int slot,
-		       double x,
-		       double y);
-void litest_touch_move_extended(struct litest_device *d,
-				unsigned int slot,
-				double x,
-				double y,
-				struct axis_replacement *axes);
-void litest_touch_down(struct litest_device *d,
-		       unsigned int slot,
-		       double x,
-		       double y);
-void litest_touch_down_extended(struct litest_device *d,
-				unsigned int slot,
-				double x,
-				double y,
-				struct axis_replacement *axes);
-void litest_touch_move_to(struct litest_device *d,
-			  unsigned int slot,
-			  double x_from, double y_from,
-			  double x_to, double y_to,
-			  int steps, int sleep_ms);
-void litest_touch_move_two_touches(struct litest_device *d,
-				   double x0, double y0,
-				   double x1, double y1,
-				   double dx, double dy,
-				   int steps, int sleep_ms);
-void litest_touch_move_three_touches(struct litest_device *d,
-				     double x0, double y0,
-				     double x1, double y1,
-				     double x2, double y2,
-				     double dx, double dy,
-				     int steps, int sleep_ms);
-void litest_hover_start(struct litest_device *d,
-			unsigned int slot,
-			double x,
-			double y);
-void litest_hover_end(struct litest_device *d, unsigned int slot);
 void litest_hover_move(struct litest_device *d,
 		       unsigned int slot,
 		       double x,
 		       double y);
-void litest_hover_move_to(struct litest_device *d,
-			  unsigned int slot,
-			  double x_from, double y_from,
-			  double x_to, double y_to,
-			  int steps, int sleep_ms);
-void litest_hover_move_two_touches(struct litest_device *d,
-				   double x0, double y0,
-				   double x1, double y1,
-				   double dx, double dy,
-				   int steps, int sleep_ms);
-void litest_button_click(struct litest_device *d,
-			 unsigned int button,
-			 bool is_press);
-void litest_button_scroll(struct litest_device *d,
-			 unsigned int button,
-			 double dx, double dy);
-void litest_keyboard_key(struct litest_device *d,
-			 unsigned int key,
-			 bool is_press);
-void litest_wait_for_event(struct libinput *li);
-void litest_wait_for_event_of_type(struct libinput *li, ...);
-void litest_drain_events(struct libinput *li);
-void litest_assert_empty_queue(struct libinput *li);
-struct libinput_event_pointer * litest_is_button_event(
-		       struct libinput_event *event,
+
+void
+litest_hover_move_to(struct litest_device *d,
+		     unsigned int slot,
+		     double x_from, double y_from,
+		     double x_to, double y_to,
+		     int steps, int sleep_ms);
+
+void
+litest_hover_move_two_touches(struct litest_device *d,
+			      double x0, double y0,
+			      double x1, double y1,
+			      double dx, double dy,
+			      int steps, int sleep_ms);
+
+void
+litest_button_click(struct litest_device *d,
+		    unsigned int button,
+		    bool is_press);
+
+void
+litest_button_scroll(struct litest_device *d,
+		     unsigned int button,
+		     double dx, double dy);
+
+void
+litest_keyboard_key(struct litest_device *d,
+		    unsigned int key,
+		    bool is_press);
+
+void
+litest_wait_for_event(struct libinput *li);
+
+void
+litest_wait_for_event_of_type(struct libinput *li, ...);
+
+void
+litest_drain_events(struct libinput *li);
+
+void
+litest_assert_empty_queue(struct libinput *li);
+
+struct libinput_event_pointer *
+litest_is_button_event(struct libinput_event *event,
 		       unsigned int button,
 		       enum libinput_button_state state);
-struct libinput_event_pointer * litest_is_axis_event(
-		       struct libinput_event *event,
-		       enum libinput_pointer_axis axis,
-		       enum libinput_pointer_axis_source source);
-struct libinput_event_pointer * litest_is_motion_event(
-		       struct libinput_event *event);
-struct libinput_event_touch * litest_is_touch_event(
-		       struct libinput_event *event,
+
+struct libinput_event_pointer *
+litest_is_axis_event(struct libinput_event *event,
+		     enum libinput_pointer_axis axis,
+		     enum libinput_pointer_axis_source source);
+
+struct libinput_event_pointer *
+litest_is_motion_event(struct libinput_event *event);
+
+struct libinput_event_touch *
+litest_is_touch_event(struct libinput_event *event,
+		      enum libinput_event_type type);
+
+struct libinput_event_keyboard *
+litest_is_keyboard_event(struct libinput_event *event,
+			 unsigned int key,
+			 enum libinput_key_state state);
+
+struct libinput_event_gesture *
+litest_is_gesture_event(struct libinput_event *event,
+			enum libinput_event_type type,
+			int nfingers);
+
+struct libinput_event_tablet_tool *
+litest_is_tablet_event(struct libinput_event *event,
 		       enum libinput_event_type type);
-struct libinput_event_keyboard * litest_is_keyboard_event(
-		       struct libinput_event *event,
-		       unsigned int key,
-		       enum libinput_key_state state);
-struct libinput_event_gesture * litest_is_gesture_event(
-		       struct libinput_event *event,
-		       enum libinput_event_type type,
-		       int nfingers);
 
-void litest_assert_button_event(struct libinput *li,
-				unsigned int button,
-				enum libinput_button_state state);
-void litest_assert_scroll(struct libinput *li,
-			  enum libinput_pointer_axis axis,
-			  int minimum_movement);
-void litest_assert_only_typed_events(struct libinput *li,
-				     enum libinput_event_type type);
+void
+litest_assert_button_event(struct libinput *li,
+			   unsigned int button,
+			   enum libinput_button_state state);
 
-struct libevdev_uinput * litest_create_uinput_device(const char *name,
-						     struct input_id *id,
-						     ...);
-struct libevdev_uinput * litest_create_uinput_abs_device(const char *name,
-							 struct input_id *id,
-							 const struct input_absinfo *abs,
-							 ...);
-#define litest_assert_double_eq(a_, b_)\
-	ck_assert_int_eq((int)(a_ * 256), (int)(b_ * 256))
+void
+litest_assert_scroll(struct libinput *li,
+		     enum libinput_pointer_axis axis,
+		     int minimum_movement);
 
-#define litest_assert_double_ne(a_, b_)\
-	ck_assert_int_ne((int)(a_ * 256), (int)(b_ * 256))
+void
+litest_assert_only_typed_events(struct libinput *li,
+				enum libinput_event_type type);
 
-#define litest_assert_double_lt(a_, b_)\
-	ck_assert_int_lt((int)(a_ * 256), (int)(b_ * 256))
+void
+litest_assert_tablet_button_event(struct libinput *li,
+				  unsigned int button,
+				  enum libinput_button_state state);
 
-#define litest_assert_double_le(a_, b_)\
-	ck_assert_int_le((int)(a_ * 256), (int)(b_ * 256))
+void
+litest_assert_tablet_proximity_event(struct libinput *li,
+				     enum libinput_tablet_tool_proximity_state state);
 
-#define litest_assert_double_gt(a_, b_)\
-	ck_assert_int_gt((int)(a_ * 256), (int)(b_ * 256))
+struct libevdev_uinput *
+litest_create_uinput_device(const char *name,
+			    struct input_id *id,
+			    ...);
 
-#define litest_assert_double_ge(a_, b_)\
-	ck_assert_int_ge((int)(a_ * 256), (int)(b_ * 256))
+struct libevdev_uinput *
+litest_create_uinput_abs_device(const char *name,
+				struct input_id *id,
+				const struct input_absinfo *abs,
+				...);
 
-void litest_timeout_tap(void);
-void litest_timeout_tapndrag(void);
-void litest_timeout_softbuttons(void);
-void litest_timeout_buttonscroll(void);
-void litest_timeout_edgescroll(void);
-void litest_timeout_finger_switch(void);
-void litest_timeout_middlebutton(void);
-void litest_timeout_dwt_short(void);
-void litest_timeout_dwt_long(void);
-void litest_timeout_gesture(void);
+void
+litest_timeout_tap(void);
 
-void litest_push_event_frame(struct litest_device *dev);
-void litest_pop_event_frame(struct litest_device *dev);
+void
+litest_timeout_tapndrag(void);
+
+void
+litest_timeout_softbuttons(void);
+
+void
+litest_timeout_buttonscroll(void);
+
+void
+litest_timeout_edgescroll(void);
+
+void
+litest_timeout_finger_switch(void);
+
+void
+litest_timeout_middlebutton(void);
+
+void
+litest_timeout_dwt_short(void);
+
+void
+litest_timeout_dwt_long(void);
+
+void
+litest_timeout_gesture(void);
+
+void
+litest_push_event_frame(struct litest_device *dev);
+
+void
+litest_pop_event_frame(struct litest_device *dev);
 
 /* this is a semi-mt device, so we keep track of the touches that the tests
  * send and modify them so that the first touch is always slot 0 and sends
@@ -456,17 +603,22 @@ struct litest_semi_mt {
 	} touches[2];
 };
 
-void litest_semi_mt_touch_down(struct litest_device *d,
-			       struct litest_semi_mt *semi_mt,
-			       unsigned int slot,
-			       double x, double y);
-void litest_semi_mt_touch_move(struct litest_device *d,
-			       struct litest_semi_mt *semi_mt,
-			       unsigned int slot,
-			       double x, double y);
-void litest_semi_mt_touch_up(struct litest_device *d,
-			     struct litest_semi_mt *semi_mt,
-			     unsigned int slot);
+void
+litest_semi_mt_touch_down(struct litest_device *d,
+			  struct litest_semi_mt *semi_mt,
+			  unsigned int slot,
+			  double x, double y);
+
+void
+litest_semi_mt_touch_move(struct litest_device *d,
+			  struct litest_semi_mt *semi_mt,
+			  unsigned int slot,
+			  double x, double y);
+
+void
+litest_semi_mt_touch_up(struct litest_device *d,
+			struct litest_semi_mt *semi_mt,
+			unsigned int slot);
 
 #ifndef ck_assert_notnull
 #define ck_assert_notnull(ptr) ck_assert_ptr_ne(ptr, NULL)
@@ -496,6 +648,30 @@ litest_disable_tap(struct libinput_device *device)
 	litest_assert_int_eq(status, expected);
 }
 
+static inline void
+litest_enable_tap_drag(struct libinput_device *device)
+{
+	enum libinput_config_status status, expected;
+
+	expected = LIBINPUT_CONFIG_STATUS_SUCCESS;
+	status = libinput_device_config_tap_set_drag_enabled(device,
+							     LIBINPUT_CONFIG_DRAG_ENABLED);
+
+	litest_assert_int_eq(status, expected);
+}
+
+static inline void
+litest_disable_tap_drag(struct libinput_device *device)
+{
+	enum libinput_config_status status, expected;
+
+	expected = LIBINPUT_CONFIG_STATUS_SUCCESS;
+	status = libinput_device_config_tap_set_drag_enabled(device,
+							     LIBINPUT_CONFIG_DRAG_DISABLED);
+
+	litest_assert_int_eq(status, expected);
+}
+
 static inline bool
 litest_has_2fg_scroll(struct litest_device *dev)
 {
