@@ -47,6 +47,18 @@ tap_default(struct libinput_device *device)
 		return "disabled";
 }
 
+static const char *
+drag_default(struct libinput_device *device)
+{
+	if (!libinput_device_config_tap_get_finger_count(device))
+		return "n/a";
+
+	if (libinput_device_config_tap_get_default_drag_enabled(device))
+		return "enabled";
+	else
+		return "disabled";
+}
+
 static const char *
 draglock_default(struct libinput_device *device)
 {
@@ -257,9 +269,13 @@ print_device_notify(struct libinput_event *ev)
 	if (libinput_device_has_capability(dev,
 					   LIBINPUT_DEVICE_CAP_TOUCH))
 		printf("touch");
+	if (libinput_device_has_capability(dev,
+					   LIBINPUT_DEVICE_CAP_TABLET_TOOL))
+		printf("tablet");
 	printf("\n");
 
 	printf("Tap-to-click:     %s\n", tap_default(dev));
+	printf("Tap-and-drag:     %s\n",  drag_default(dev));
 	printf("Tap drag lock:    %s\n", draglock_default(dev));
 	printf("Left-handed:      %s\n", left_handed_default(dev));
 	printf("Nat.scrolling:    %s\n", nat_scroll_default(dev));
